///////////////////////////////////////////////////////////////////////////////
// File: SocketServer.cpp
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2009 JetByte Limited.
//
// This software is provided "as is" without a warranty of any kind. All
// express or implied conditions, representations and warranties, including
// any implied warranty of merchantability, fitness for a particular purpose
// or non-infringement, are hereby excluded. JetByte Limited and its licensors
// shall not be liable for any damages suffered by licensee as a result of
// using the software. In no event will JetByte Limited be liable for any
// lost revenue, profit or data, or for direct, indirect, special,
// consequential, incidental or punitive damages, however caused and regardless
// of the theory of liability, arising out of the use of or inability to use
// software, even if JetByte Limited has been advised of the possibility of
// such damages.
//
///////////////////////////////////////////////////////////////////////////////

#include "JetByteTools/Admin/Admin.h"

#include "SocketServer.h"

#include "JetByteTools/CoreTools/FileUtils.h"
#include "JetByteTools/CoreTools/StringUtils.h"
#include "JetByteTools/CoreTools/DebugTrace.h"
#include "JetByteTools/CoreTools/Exception.h"

#include "JetByteTools/IOTools/IBuffer.h"

#include "JetByteTools/SocketTools/IStreamSocket.h"

#pragma hdrstop

///////////////////////////////////////////////////////////////////////////////
// Using directives
///////////////////////////////////////////////////////////////////////////////

using JetByteTools::Core::Output;
using JetByteTools::Core::OutputEx;
using JetByteTools::Core::DumpData;
using JetByteTools::Core::CException;
using JetByteTools::Core::GetStringLength;

using JetByteTools::IO::IAllocateBuffers;
using JetByteTools::IO::CSmartBuffer;
using JetByteTools::IO::IIOPool;

using JetByteTools::Socket::CStreamSocketServerEx;
using JetByteTools::Socket::IStreamSocket;
using JetByteTools::Socket::IAllocateStreamSockets;
using JetByteTools::Socket::IFullAddress;
using JetByteTools::Socket::IAddress;
using JetByteTools::Socket::ILimitConnections;
using JetByteTools::Socket::ListenBacklog;
using JetByteTools::Socket::NoZeroByteRead;

using std::string;

///////////////////////////////////////////////////////////////////////////////
// CSocketServer
///////////////////////////////////////////////////////////////////////////////

CSocketServer::CSocketServer(
   const string &welcomeMessage,
   const IFullAddress &address,
   const ListenBacklog listenBacklog,
   IIOPool &pool,
   IAllocateStreamSockets &socketAllocator,
   IAllocateBuffers &bufferAllocator,
   ILimitConnections &connectionLimiter)
   :  CStreamSocketServerEx(address, listenBacklog, *this, pool, socketAllocator, bufferAllocator, NoZeroByteRead, connectionLimiter),
      m_welcomeMessage(welcomeMessage)
{
}

CSocketServer::~CSocketServer()
{
   WaitForShutdownToComplete();
}

void CSocketServer::OnConnectionEstablished(
   IStreamSocket &socket,
   const IAddress & /*address*/)
{
   Output(_T("OnConnectionEstablished"));

   socket.Write(m_welcomeMessage.c_str(), GetStringLength<DWORD>(m_welcomeMessage));

   socket.TryRead();
}

void CSocketServer::OnReadCompleted(
   IStreamSocket &socket,
   CSmartBuffer &buffer)
{
   EchoMessage(socket, buffer);

   socket.TryRead();
}

void CSocketServer::EchoMessage(
   IStreamSocket &socket,
   CSmartBuffer &buffer) const
{
   DEBUG_ONLY(Output(_T("\r\n") + DumpData(buffer->GetMemory(), buffer->GetUsed(), 60, true)));

   socket.Write(buffer);
}

///////////////////////////////////////////////////////////////////////////////
// End of file: SocketServer.cpp
///////////////////////////////////////////////////////////////////////////////
