#pragma once
#ifndef ECHO_SERVER_SERVICE_INCLUDED__
#define ECHO_SERVER_SERVICE_INCLUDED__
///////////////////////////////////////////////////////////////////////////////
// File: Service.h
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2004 JetByte Limited.
//
// This software is provided "as is" without a warranty of any kind. All
// express or implied conditions, representations and warranties, including
// any implied warranty of merchantability, fitness for a particular purpose
// or non-infringement, are hereby excluded. JetByte Limited and its licensors
// shall not be liable for any damages suffered by licensee as a result of
// using the software. In no event will JetByte Limited be liable for any
// lost revenue, profit or data, or for direct, indirect, special,
// consequential, incidental or punitive damages, however caused and regardless
// of the theory of liability, arising out of the use of or inability to use
// software, even if JetByte Limited has been advised of the possibility of
// such damages.
//
///////////////////////////////////////////////////////////////////////////////

#include "JetByteTools/ServiceTools/Service.h"
#include "JetByteTools/ServiceTools/ServiceCallbacks.h"

#include "ServerCommon/SimpleServiceStartupEventHandler.h"

///////////////////////////////////////////////////////////////////////////////
// Classes defined in other files...
///////////////////////////////////////////////////////////////////////////////

namespace JetByteTools
{
   namespace IO
   {
      class CIOPool;
      class CBufferAllocator;
   }

   namespace Socket
   {
      class CSequencedStreamSocketAllocator;
      class CConnectionLimiter;
   }
}

class CSocketServer;

///////////////////////////////////////////////////////////////////////////////
// CService
///////////////////////////////////////////////////////////////////////////////

class CService :
   public JetByteTools::Service::CService,
   private JetByteTools::Service::CServiceCallbacks,
   private CSimpleServiceStartupEventHandler
{
   public :

      CService();

      CService(
         const CService &rhs) = delete;

      CService &operator=(
         const CService &rhs) = delete;

      virtual ~CService() = default;

   private :

      // Implement IServiceCallbacks interface

      JetByteTools::Service::ServiceTypes::ExitCode InitialiseService(
         JetByteTools::Service::IShutdownService &shutdown,
         JetByteTools::Service::INotifyProgress &notify,
         JetByteTools::Service::IManageServiceNotifications &notificationManager,
         DWORD argc,
         const PTSTR *ppArgv) override;

      void ContinueService(
         JetByteTools::Service::INotifyProgress &notify) override;

      void PauseService(
         JetByteTools::Service::INotifyProgress &notify) override;

      void StopService(
         JetByteTools::Service::INotifyProgress &notify,
         JetByteTools::Service::ServiceTypes::StopReason stopReason) override;

      JetByteTools::Service::ServiceTypes::ExitCode UninitialiseService(
         JetByteTools::Service::INotifyProgress &notify) override;

      JetByteTools::Service::ServiceTypes::ExitCode OnException(
         const JetByteTools::Core::CException &e) override;

      JetByteTools::Service::ServiceTypes::ExitCode OnException(
         const JetByteTools::Win32::CSEHException &e) override;

      JetByteTools::Service::ServiceTypes::ExitCode OnException(
         const std::exception &e) override;

      JetByteTools::Service::ServiceTypes::ExitCode OnException() override;

      JetByteTools::IO::CIOPool *m_pPool;

      JetByteTools::Socket::CSequencedStreamSocketAllocator *m_pSocketAllocator;

      JetByteTools::IO::CBufferAllocator *m_pBufferAllocator;

      JetByteTools::Socket::CConnectionLimiter *m_pLimiter;

      CSocketServer *m_pServer;

      bool m_isActive;
};

#endif // ECHO_SERVER_SERVICE_INCLUDED__

///////////////////////////////////////////////////////////////////////////////
// End of file: Service.h
///////////////////////////////////////////////////////////////////////////////
