///////////////////////////////////////////////////////////////////////////////
// File: StreamSocketAllocator.cpp
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2005 JetByte Limited.
//
// This software is provided "as is" without a warranty of any kind. All
// express or implied conditions, representations and warranties, including
// any implied warranty of merchantability, fitness for a particular purpose
// or non-infringement, are hereby excluded. JetByte Limited and its licensors
// shall not be liable for any damages suffered by licensee as a result of
// using the software. In no event will JetByte Limited be liable for any
// lost revenue, profit or data, or for direct, indirect, special,
// consequential, incidental or punitive damages, however caused and regardless
// of the theory of liability, arising out of the use of or inability to use
// software, even if JetByte Limited has been advised of the possibility of
// such damages.
//
///////////////////////////////////////////////////////////////////////////////

#include "JetByteTools/Admin/Admin.h"

#include "StreamSocketAllocator.h"

#include "JetByteTools/CoreTools/FileUtils.h"
#include "JetByteTools/CoreTools/DebugTrace.h"

#pragma hdrstop

///////////////////////////////////////////////////////////////////////////////
// Using directives
///////////////////////////////////////////////////////////////////////////////

using JetByteTools::Core::Output;

using JetByteTools::Socket::IMonitorSocketAllocation;
using JetByteTools::Socket::PerSocketBufferPoolingLimit;
using JetByteTools::Socket::ConsecutiveSocketOperations;

///////////////////////////////////////////////////////////////////////////////
// CStreamSocketAllocator
///////////////////////////////////////////////////////////////////////////////

CStreamSocketAllocator::CStreamSocketAllocator(
   const SocketCount maxFreeSockets,
   const PerSocketBufferPoolingLimit maxPooledBuffers,
   const ConsecutiveSocketOperations maxConsecutiveOperations,
   const bool log)
   :  JetByteTools::Socket::CStreamSocketAllocator(maxFreeSockets, maxPooledBuffers, maxConsecutiveOperations),
      m_log(log)
{

}

CStreamSocketAllocator::CStreamSocketAllocator(
   IMonitorSocketAllocation &monitor,
   const SocketCount maxFreeSockets,
   const PerSocketBufferPoolingLimit maxPooledBuffers,
   const ConsecutiveSocketOperations maxConsecutiveOperations,
   const bool log)
   :  JetByteTools::Socket::CStreamSocketAllocator(monitor, maxFreeSockets, maxPooledBuffers, maxConsecutiveOperations),
      m_log(log)
{

}

CStreamSocketAllocator::~CStreamSocketAllocator()
{
   // Must ensure that we have been flushed before we call the base class's
   // destructor otherwise it may try and call into us (as a monitor) when
   // it deletes sockets...

   ReleaseSockets();
}

void CStreamSocketAllocator::OnSocketCreated()
{
   if (m_log)
   {
      Output(_T("OnSocketCreated"));
   }
}

void CStreamSocketAllocator::OnSocketAttached(
   JetByteTools::Core::IIndexedOpaqueUserData & /*userData*/)
{
   if (m_log)
   {
      Output(_T("OnSocketAttached"));
   }
}

void CStreamSocketAllocator::OnSocketReleased(
   JetByteTools::Core::IIndexedOpaqueUserData & /*userData*/)
{
   if (m_log)
   {
      Output(_T("OnSocketReleased"));
   }
}

void CStreamSocketAllocator::OnSocketDestroyed()
{
   if (m_log)
   {
      Output(_T("OnSocketDestroyed"));
   }
}

///////////////////////////////////////////////////////////////////////////////
// End of file: StreamSocketAllocator.cpp
///////////////////////////////////////////////////////////////////////////////
