@Echo off

REM Most arguments passed to us via the environment as they contain spaces...

if not defined TEST_NAME goto NotSet
if not defined SERVER goto NotSet
if not defined SERVICE_NAME goto NotSet
if not defined OUT_DIR goto NotSet
if not defined TEST_CLIENT goto NotSet
if not defined TEST_CLIENT_ARGS goto NotSet

goto DisplayDebug

:NotSet
Echo One or more of the following variables is not set:
Echo -
Echo TEST_NAME=%TEST_NAME%
Echo SERVER=%SERVER%
Echo SERVICE_NAME=%SERVICE_NAME%
Echo TEST_CLIENT=%TEST_CLIENT%
Echo TEST_CLIENT_ARGS=%TEST_CLIENT_ARGS%
Echo -
Echo Please ensure they are all set in the environment before calling.
Exit /b 1

:DisplayDebug

if not defined JETBYTE_DEBUG_BUILD_SCRIPTS goto NoDebug

Echo Script debug information - RunTest.bat
Echo - 
Echo Required: TEST_NAME=%TEST_NAME%
Echo Required: SERVER=%SERVER%
Echo Required: SERVICE_NAME=%SERVICE_NAME%
Echo Required: OUT_DIR=%OUT_DIR%
Echo Required: TEST_CLIENT=%TEST_CLIENT%
Echo Required: TEST_CLIENT_ARGS=%TEST_CLIENT_ARGS%
Echo Required: SERVER_DIR=%SERVER_DIR%
Echo Optional: SERVICE_INSTANCE_NAME=%SERVICE_INSTANCE_NAME%
Echo Optional: SUPPORTS_PERF_COUNTERS=%SUPPORTS_PERF_COUNTERS%
Echo Optional: CHANGE_TO_WORKING_DIR=%CHANGE_TO_WORKING_DIR%
Echo Optional: WORKING_DIR_IS_SERVER_DIR=%WORKING_DIR_IS_SERVER_DIR%
Echo Optional: TEST_RUNNER=%TEST_RUNNER%
Echo -

:NoDebug

setlocal

Echo Starting Test: %TEST_NAME%
Echo -

if not defined SERVER_DIR set SERVER_DIR=%OUT_DIR%

if not defined TEST_RUNNER goto InstallService

Echo We can't run service tests under test runners.
Echo Since there are multiple test runners configured we only run
Echo the service test once when the test runner is "TestRunner.exe"

if %TEST_RUNNER% == ..\..\Test\TestRunner.exe goto InstallService

Echo Skipping test: TEST_RUNNER=%TEST_RUNNER%

goto End

:InstallService

if defined CHANGE_TO_WORKING_DIR pushd %CHANGE_TO_WORKING_DIR%

if defined WORKING_DIR_IS_SERVER_DIR set SERVER_DIR=.\

Echo Installing Service: %SERVER_DIR%\%SERVER%

if not defined SUPPORTS_PERF_COUNTERS goto AfterPerfCounterInstall

REM perf counters for non instance services are installed as part of the normal
REM service installation routine with -install

if not defined SERVICE_INSTANCE_NAME goto AfterPerfCounterInstall

REM First we attempt to remove the counters in case they're already installed!
%SERVER_DIR%\%SERVER% -noMessages -removeCounters %SERVICE_INSTANCE_NAME%

%SERVER_DIR%\%SERVER% -noMessages -installCounters %SERVICE_INSTANCE_NAME%

:AfterPerfCounterInstall

REM First we attempt to remove the service in case it's already installed!
%SERVER_DIR%\%SERVER% -noMessages -remove %SERVICE_INSTANCE_NAME%

REM %SERVER_DIR%\%SERVER% -install %SERVICE_INSTANCE_NAME%
%SERVER_DIR%\%SERVER% -noMessages -install %SERVICE_INSTANCE_NAME%
IF ERRORLEVEL 5 goto FailedToInstall
IF ERRORLEVEL 4 goto InsufficientRights
IF ERRORLEVEL 1 goto FailedToInstall

Echo Starting Service: %SERVICE_NAME%

net start %SERVICE_NAME%
IF ERRORLEVEL 1 goto FailedToStart

if defined CHANGE_TO_WORKING_DIR popd

Echo Waiting for server start up to complete
..\..\Test\ServerShutdown.exe WAIT_FOR_START
IF ERRORLEVEL 1 goto FailedToStart

Echo Service: %SERVICE_NAME% running
Echo -

Echo Starting client
Echo -

%TEST_CLIENT% %TEST_CLIENT_ARGS%
IF ERRORLEVEL 1 goto ClientFailed

Echo Client completed
Echo -

Echo Shutting down server
Echo -

Echo Test Passed: %TEST_NAME%
Echo -

goto StopRemoveAndExit

:ClientFailed

Echo Test Failed: %TEST_NAME%
Echo Client failed.
Echo ERRORLEVEL=%ERRORLEVEL%
Echo -

goto ForceShutdown

:TestFailed

Echo Test Failed: %TEST_NAME%
Echo Server failed.
Echo ERRORLEVEL=%ERRORLEVEL%
Echo -

:ForceShutdown

Echo Attempting to force the server to shut down
Echo -

net stop %SERVICE_NAME%
IF not ERRORLEVEL 1 set FATAL_ERROR_EXIT=%ERRORLEVEL%

..\..\Test\ServerShutdown.exe WAIT_FOR_STOP

goto RemoveAndExit

:FailedToInstall

Echo Test Failed: %TEST_NAME%
Echo Server Failed to install service: %SERVICE_NAME% - %SERVER_DIR%\%SERVER%
Echo ERRORLEVEL=%ERRORLEVEL%

Exit /b 1

:FailedToStart

Echo Test Failed: %TEST_NAME%
Echo Server Failed to start: %SERVICE_NAME% - %SERVER_DIR%\%SERVER%
Echo ERRORLEVEL=%ERRORLEVEL%

set FATAL_ERROR_EXIT=%ERRORLEVEL%
goto RemoveAndExit

:FailedToStop

Echo Test Failed: %TEST_NAME%
Echo Server Failed to stop: %SERVICE_NAME% - %SERVER_DIR%\%SERVER%
Echo ERRORLEVEL=%ERRORLEVEL%

goto ForceShutdown

:FailedToRemove

Echo Test Failed: %TEST_NAME%
Echo Server Failed to remove service: %SERVICE_NAME% - %SERVER_DIR%\%SERVER%
Echo ERRORLEVEL=%ERRORLEVEL%

Exit /b 1

:InsufficientRights

Echo Test Skipped: %TEST_NAME%
Echo Can't run service tests due to insufficient user rights

Exit /b 0

:StopRemoveAndExit

net stop %SERVICE_NAME%
IF not ERRORLEVEL 1 goto RemoveAndExit

Echo Failed to stop service: %SERVICE_NAME%
set FATAL_ERROR_EXIT=ERRORLEVEL

:RemoveAndExit

%SERVER_DIR%\%SERVER% -noMessages -remove %SERVICE_INSTANCE_NAME%
IF not ERRORLEVEL 1 goto RemovePerfCounters

Echo Failed to remove service: %SERVICE_NAME%
set FATAL_ERROR_EXIT=ERRORLEVEL

:RemovePerfCounters

if not defined SUPPORTS_PERF_COUNTERS goto AfterPerfCounterRemove

REM perf counters for non instance services are installed as part of the normal
REM service installation routine with -install

if not defined SERVICE_INSTANCE_NAME goto AfterPerfCounterRemove

%SERVER_DIR%\%SERVER% -noMessages -removeCounters %SERVICE_INSTANCE_NAME%
IF not ERRORLEVEL 1 goto AfterPerfCounterRemove

Echo Failed to remove perf counters for service: %SERVICE_NAME%
set FATAL_ERROR_EXIT=ERRORLEVEL

:AfterPerfCounterRemove

if defined FATAL_ERROR_EXIT Exit /b %FATAL_ERROR_EXIT%

:End
